
/*
 * WPA Supplicant - Layer2 packet handling
 * Copyright (c) 2003-2004, Jouni Malinen <jkmaline@cc.hut.fi>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Alternatively, this software may be distributed under the terms of BSD
 * license.
 *
 * See README and COPYING for more details.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <errno.h>
#include <unistd.h>

#include "common.h"
#include "eloop.h"
#include "l2pos2.h"
#include "hos2.h"
#include "poid.hpp"

// 0060B314E7FE

struct l2_packet_data {

        int fd; /* packet socket for EAPOL frames */
        char ifname[100];
        u8 own_addr[ETH_ALEN];
        void (*rx_callback)(void *ctx, unsigned char *src_addr,
                            unsigned char *buf, size_t len);
        void *rx_callback_ctx;
        int rx_l2_hdr; /* whether to include layer 2 (Ethernet) header in calls
                        * to rx_callback */
};


int l2_packet_get_own_addr(struct l2_packet_data *l2, u8 *addr)
{
//      printf("XXXXXXXXXXXXXXXXXX %s\n",__FUNCTION__);
        memcpy(addr, l2->own_addr, ETH_ALEN);
        return 0;
}


void l2_packet_set_rx_l2_hdr(struct l2_packet_data *l2, int rx_l2_hdr)
{
        l2->rx_l2_hdr = 1 ;// rx_l2_hdr;
}

int l2_packet_send(struct l2_packet_data *l2, u8 *buf, size_t len)
{
//      return pcap_sendpacket(l2->pcap, buf, len);
//      printf("XXXXXXXXXXXXXXXXXX %s\n",__FUNCTION__);

/*
        buf[0] = 0x00 ;
        buf[1] = 0x60 ;
        buf[2] = 0xB3 ;
        buf[3] = 0x14 ;
        buf[4] = 0xE7 ;
        buf[5] = 0xFE ;
*/


    _Os2SetOid(l2->fd,OID_PRIVATE_WRAPPER_802_1X_TX,(char*)buf,len);

    return 0 ;
}

static u_char BUFFER[2000] ;

static void l2_packet_receive(int sock, void *eloop_ctx, void *sock_ctx)
{
//      printf("XXXXXXX %s [0x%08x]\n",__FUNCTION__,sock);
        struct l2_packet_data *l2 = eloop_ctx;
        const u_char *packet;
        struct l2_ethhdr *ethhdr;
        unsigned char *buf;
        size_t len,rxlen;

        // XXXXpacket = pcap_next(pcap, &hdr);

        int UL = 0;
        _Os2GetOid(sock,OID_PRIVATE_WRAPPER_802_1X_READY,(char*)&UL,4);
        // printf("RXREADY : %d\n",UL);

        if(UL == 0)return ;

        rxlen = -1 ;

        if( UL ){

            rxlen = _Os2GetOid(sock,OID_PRIVATE_WRAPPER_802_1X_RX,(char*)BUFFER,2000);
            if( rxlen ){
                //printf("XXXX rxlen : %d\n",rxlen);
            }
        }

        if( rxlen <= 0 )return ;

        packet = BUFFER ;

        buf = &BUFFER[6] ;
/*
        buf[0] = 0x00 ;
        buf[1] = 0x60 ;
        buf[2] = 0xB3 ;
        buf[3] = 0x14 ;
        buf[4] = 0xE7 ;
        buf[5] = 0xFE ;
*/

        ethhdr = (struct l2_ethhdr *) packet;

        printf("XXXX rx_l2_hdr %d\n",l2->rx_l2_hdr);

        if (l2->rx_l2_hdr) {
                buf = (unsigned char *) ethhdr;
                len = rxlen;
        } else {
                buf = (unsigned char *) (ethhdr + 1);
                len = rxlen - sizeof(*ethhdr);
        }
        l2->rx_callback(l2->rx_callback_ctx, ethhdr->h_source, buf, len);
}


static void l2_packet_receive_timeout(void *eloop_ctx, void *timeout_ctx)
{
    //  printf("XXXXXXXXXXXXXXXXXX %s\n",__FUNCTION__);
        struct l2_packet_data *l2 = eloop_ctx;

        void *pcap = timeout_ctx;

        /* Register new timeout before calling l2_packet_receive() since
         * receive handler may free this l2_packet instance (which will
         * cancel this timeout). */

        eloop_register_timeout(0, 100000, l2_packet_receive_timeout,
                               l2, pcap);
        l2_packet_receive(l2->fd, eloop_ctx, timeout_ctx);


}



struct l2_packet_data * l2_packet_init(
        const char *ifname, const u8 *own_addr, unsigned short protocol,
        void (*rx_callback)(void *ctx, unsigned char *src_addr,
                            unsigned char *buf, size_t len),
        void *rx_callback_ctx)
{

        //printf("XXXXXXXXXXXXXXXXXX %s %s\n",__FUNCTION__,ifname);
        struct l2_packet_data *l2;

        l2 = malloc(sizeof(struct l2_packet_data));
        if (l2 == NULL)
                return NULL;
        memset(l2, 0, sizeof(*l2));
        strncpy(l2->ifname, ifname, sizeof(l2->ifname));
        l2->rx_callback = rx_callback;
        l2->rx_callback_ctx = rx_callback_ctx;

        l2->rx_l2_hdr = 0;

        l2->fd = _Os2OpenDev(l2->ifname);
        long Dummy = 1 ;
        _Os2SetOid(l2->fd,OID_PRIVATE_WRAPPER_802_1X_STATE,(char*)&Dummy,4);

        int rxp ;
        do{
            _Os2GetOid(l2->fd,OID_PRIVATE_WRAPPER_802_1X_READY,(char*)&rxp,4);
            if( rxp )_Os2GetOid(l2->fd,OID_PRIVATE_WRAPPER_802_1X_RX,(char*)BUFFER,2000);
        }while(rxp);

        if (own_addr)memcpy(l2->own_addr, own_addr, ETH_ALEN);

        eloop_register_timeout(0, 100000, l2_packet_receive_timeout,
                               l2, l2);


        return l2;
}


void l2_packet_deinit(struct l2_packet_data *l2)
{
        //printf("XXXXXXXXXXXXXXXXXX %s\n",__FUNCTION__);
        if (l2 == NULL)return;
        free(l2);
}

